"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""

from game_objects import Robot
from animators import moving_box_animator, tile_animator, wall_animator, spawn_animator
import tile_objects as TO
import rendering_utils
import tokens
import animators

class Level:
    new_robot = 0
    def __init__(self, engine):
        """
        The engine object is made available to
        the level to allow the level to fire an
        event to the engine to change the level.
        """
        self.renderableList = []
        self.tiles = []
        self.engine = engine
        self.spawns = []
        self.executables = []
        self.tick = 0
        self.width = 0
        self.height = 0
        self.complete = False
        self.display_list = None
        self.available_token_size = 0
        
        self.tutorial_steps = []
        self.tutorial_count = 0

        self.remaining_robots = 5
    def initialiseMap(self):

        self.available_tokens = []
        for i in range(self.width):
            row = []
            self.tiles.append(row)
            for j in range(self.height):
                tile = TO.Tile([i,j])
                row.append(tile)
                self.renderableList.append(tile)
             
        for i in range(self.width):
            for j in range(self.height):
                if i > 0:
                    self.tiles[i][j].left = self.tiles[i-1][j]
                if i < self.width - 1:
                    self.tiles[i][j].right = self.tiles[i+1][j]
                if j > 0:
                    self.tiles[i][j].top = self.tiles[i][j-1]
                if j < self.height - 1:
                    self.tiles[i][j].bottom = self.tiles[i][j+1]
        
        self.available_tokens.append (tokens.ForwardCommand(1))
        self.available_tokens.append (tokens.LeftCommand())
        self.available_tokens.append (tokens.RightCommand())
        if self.engine.currentLevel > 3:
            self.available_tokens.append (tokens.ForwardCommand(5))
            if self.engine.currentLevel > 5:
                self.available_tokens.append (tokens.PushCommand())
                if self.engine.currentLevel > 9:
                    self.available_tokens.append (tokens.SeekCommand())
                    if self.engine.currentLevel > 12:
                        self.available_tokens.append (tokens.HackCommand())

        for t in self.available_tokens:
            self.available_token_size += t.length

    def destroyMap(self):
        self.width = 0
        self.height = 0
        self.tiles = []

    def addSpawnPoint(self,spawn):
        self.spawns.append(spawn)

    def setWidth(self, width):
        self.width = width
        if self.width and self.height:
            self.initialiseMap()

    def setHeight(self, height):
        self.height = height
        if self.width and self.height:
            self.initialiseMap()

    def setSize(self, width, height):
        self.width = width
        self.height = height
        self.initialiseMap()

    def setMapElement(self, tile, pos):
        """
        Add a tile to the map. Pos is a tuple and tile is one of the tileable
        objects.
        """
        x, y = pos
        
        #print x,y
        #print self.width,self.height
        
        oldTile = self.tiles[x][y]
        self.renderableList.remove(oldTile)
        if x < self.width and y < self.height:
            self.tiles[x][y] = tile
            if x > 0:
                self.tiles[x - 1][y].right = tile   
                tile.left = self.tiles[x - 1][y]
            if x < self.width - 1:
                self.tiles[x + 1][y].left = tile 
                tile.right = self.tiles[x + 1][y]
            if y > 0:
                self.tiles[x][y - 1].top = tile
                tile.bottom = self.tiles[x][y - 1]
            if y < self.height - 1:
                self.tiles[x][y + 1].bottom = tile
                tile.top = self.tiles[x][y + 1]
            self.renderableList.append(tile)
            if (tile.executable()):
                self.executables.append(tile)

    def getMapElement(self, pos):
        x, y = pos
        return self.tiles[x][y]

    def execute(self):
        for obj in self.executables:
            obj.execute()
                           
            if (self.complete):
                break
        for obj in self.executables:
            obj.complete()
        self.tick += 1

    def render(self,sub_ticks,phase):
        rendering_utils.callDisplayList(self.bg_list)
        for obj in self.renderableList:
            obj.anim.draw (phase, sub_ticks)
    
    def doTutorialStep(self):
        if len(self.tutorial_steps) == 0:
            return
        function, args = self.tutorial_steps[self.tutorial_count]
        
        func = getattr(self,"tutorial_"+function)
        result = func(args)
        
        self.tutorial_count += 1
        if self.tutorial_count >= len(self.tutorial_steps):
            self.tutorial_count = 0
        
        if result:
            self.doTutorialStep()
    
    def tutorial_robot(self,args):
        commandstr,index = args
        spawn = self.spawns[index]
        spawn.activated = True

        robot = self.add_robot(spawn.pos)
        robot.set_commands(commandstr)
 
        robot.created = True
        robot.spawn = spawn
        robot.dir = spawn.dir

        self.engine.selected_spawn = spawn
        self.engine.new_robot = robot
    
        self.launch_robot(robot)
    
        return False
    
    def tutorial_disable(self,index):
        self.spawns[index].activated = False   
        
        return True
    
    def add_robot (self,pos):
        r = Robot(self,pos) 
        self.renderableList.append(r)
        return r

    def remove_robot(self, robot, nextStep = True):
        self.renderableList.remove(robot)
        self.executables.remove(robot)
        
        if self.engine.tutorial and nextStep:
            self.doTutorialStep()

    def add_enemy(self, enemy):
        self.renderableList.append(enemy)
        self.executables.append(enemy)

    def launch_robot (self,r):
        self.executables.append(r)

    def pre_render(self):
        for obj in self.renderableList:
            obj.pre_anim.draw(None, None)

    def finalise(self):
        #print "finalised level"
        for obj in self.renderableList:
            obj.finalize()
        self.bg_list = rendering_utils.generateDisplayList(self.pre_render)

    def compile(self):
        for row in self.tiles:
            for tile in row:
                tile.compile(self)
