"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
import re
import renderer
import pygame
import math

def cross(u,v):
    """input two vectors u and v in 3-D space,
    output a cross product of vector w, in column or in row
    accordingly."""
    ppp1,ppp2,ppp3=0.0,0.0,0.0
    ppp1=u[1]*v[2]-u[2]*v[1]
    ppp2=u[2]*v[0]-u[0]*v[2]
    ppp3=u[0]*v[1]-u[1]*v[0]
    # store the result of the cross product in u
    u[0]=ppp1
    u[1]=ppp2
    u[2]=ppp3
    return u #return the cross product of vector u x v.     

def cross2(u,v):
    return u[0]*v[1]-u[1]*v[0]
    
def dot2(u,v):
    return u[0]*v[0] + u[1]*v[1] 
def length2(u):
    return math.sqrt (u[0]*u[0] + u[1]*u[1] )
def normalize2(u):
    l = length2(u)
    if (l==0):
        return [1,0]
    return [u[0]/l,u[1]/l]
def angle (u,v):
    un = normalize2(u)
    vn = normalize2(v)
    c = cross2 (un,vn)
    dot = dot2(normalize2(u),normalize2(v))
    if (dot > 1):
        dot = 1
    if (dot < -1):
        dot = -1
    angle = math.acos (dot)
    if (c<0):
        angle *= -1
    return angle
    
def subtract (va,vb):
    return [a-b for a,b in zip(va,vb)]
def length (v):
    return math.sqrt (v[0]*v[0] + v[1]*v[1] + v[2]*v[2])
def normalized_cross(u,v):
    c = cross (u,v)
    
    l = length(c)
    return [a / l for a in c]
    
def draw_rectangle(sX, sY, eX, eY):
    glDisable(GL_TEXTURE_2D)
    glDisable(GL_LIGHTING)
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
    glRectf(sX, sY, eX, eY)
    glEnable(GL_TEXTURE_2D)
    glEnable(GL_LIGHTING)

def stroke_rectangle(sX, sY, eX, eY):
    glDisable(GL_TEXTURE_2D)
    glDisable(GL_LIGHTING)
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
    glBegin(GL_POLYGON)
    glVertex2f(sX, sY)
    glVertex2f(eX, sY)
    glVertex2f(eX, eY)
    glVertex2f(sX, eY)
    glEnd()
    glEnable(GL_TEXTURE_2D)
    glEnable(GL_LIGHTING)
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)

def renderQuad(sX, sY, eX, eY, height = 0, texturePacking = (1, 1)):
    """
    Quad is assumed to be on the level floor, giving the following
    assumptions:
     - The surface normal is straight up.
     - All setup has been done previously
    """
    
    glBegin(GL_QUADS)
    glNormal3f(0.0, 0.0, 1.0)
    packX, packY = texturePacking

    glTexCoord2f(0.0, packY)
    glVertex3f(sX, sY,height)
    glTexCoord2f(packX, packY)
    glVertex3f(eX, sY, height)
    glTexCoord2f(packX, 0.0)
    glVertex3f(eX, eY, height)
    glTexCoord2f(0.0, 0.0)
    glVertex3f(sX, eY, height)
    glEnd()
    
def draw_offset_square (dim):
    glPushMatrix ()
    glTranslatef (0,0,dim[2]/2.0);
    renderQuad(-dim[0]/2.0,-dim[1]/2.0,dim[0]/2.0,dim[1]/2.0);
    glPopMatrix ()



def renderBox(pos, dim ):
    glPushMatrix ()
    glTranslatef (*pos)
    draw_offset_square (dim)
    glRotatef (90,1,0,0)
    draw_offset_square ([dim[0],dim[2],dim[1]])
    glRotatef (180,1,0,0)
    draw_offset_square ([dim[0],dim[2],dim[1]])
    glRotatef (90,1,0,0)
    glRotatef (90,0,1,0)
    draw_offset_square ([dim[2],dim[1],dim[0]])
    glRotatef (180,0,1,0)
    draw_offset_square ([dim[2],dim[1],dim[0]])
    glPopMatrix()
    
def renderPartialBox(pos, dim ,sides):
    glPushMatrix ()
    glTranslatef (*pos)
    draw_offset_square (dim)
    glRotatef (90,1,0,0)
    if (sides[3]):
       draw_offset_square ([dim[0],dim[2],dim[1]])
    glRotatef (180,1,0,0)
    if (sides[1]):
       draw_offset_square ([dim[0],dim[2],dim[1]])
    glRotatef (90,1,0,0)
    glRotatef (90,0,1,0)
    if (sides[0]):
        draw_offset_square ([dim[2],dim[1],dim[0]])
    glRotatef (180,0,1,0)
    if (sides[2]):
        draw_offset_square ([dim[2],dim[1],dim[0]])
    glPopMatrix()
    
def renderPointer(pos, dim,color ):
    glColor (*color)
    glDisable(GL_TEXTURE_2D)
    glPushMatrix ()
    glTranslatef (*pos)
    bottom_point = [0,0,0]
    fl = [-dim[0]/2.0, -dim[1]/2.0,dim[2]]
    fr = [dim[0]/2.0, -dim[1]/2.0,dim[2]]
    bl = [-dim[0]/2.0, dim[1]/2.0,dim[2]]
    br = [dim[0]/2.0, dim[1]/2.0,dim[2]]
    
    #bt = (0,0)
    #flt = (1,0)
    #frt = (1,1)
    #blt = (1,1)
    #brt = (1,0)
    
    
    glBegin(GL_TRIANGLES)
    
    glNormal3f(0.0, -1, -1)#todo make this right
    #glTexCoord2f(*bt)
    glVertex3f(*bottom_point)
    #glTexCoord2f(*frt)
    glVertex3f(*fr)
    #glTexCoord2f(*flt)
    glVertex3f(*fl)

    glNormal3f(-1, 0, -1)#todo make this right
    #glTexCoord2f(*bt)
    glVertex3f(*bottom_point)
    glVertex3f(*fl)
    glVertex3f(*bl)

    glNormal3f(0.0, 1, -1)#todo make this right
    #glTexCoord2f(*bt)
    glVertex3f(*bottom_point)
    glVertex3f(*bl)
    glVertex3f(*br)

    glNormal3f(1, 0, -1)#todo make this right
    #glTexCoord2f(*bt)
    glVertex3f(*bottom_point)
    glVertex3f(*br)
    glVertex3f(*fr)
    
    glNormal3f(0.0, 0, 1)
    glVertex3f(*fr)
    glVertex3f(*bl)
    glVertex3f(*fl)
    glVertex3f(*fr)
    glVertex3f(*br)
    glVertex3f(*bl)
    glEnd()
    glPopMatrix()
    glEnable(GL_TEXTURE_2D)

def renderExplosion(particles, debris):
    glDisable(GL_LIGHTING)
    glDisable(GL_DEPTH_TEST) # WTF!!!
    
    glBegin(GL_POINTS)
    for particle in particles:
        glColor(*particle.colour)
        glVertex3f(*particle.position)
    glEnd()

    glEnable(GL_LIGHTING)
    glEnable(GL_DEPTH_TEST)

    glNormal3f(0.0, 0.0, 1.0)

    for debri in debris:
        glColor(*debri.colour)
        glPushMatrix()

        glTranslatef(*debri.position)
        glRotatef(debri.orientation[0], 1.0, 0.0, 0.0)
        glRotatef(debri.orientation[1], 0.0, 1.0, 0.0)
        glRotatef(debri.orientation[2], 0.0, 0.0, 1.0)

        glScalef(*debri.scale)

        glBegin(GL_TRIANGLES)
        glVertex3f(0.0, 0.5, 0.0)
        glVertex3f(-0.25, 0.0, 0.0)
        glVertex3f(0.25, 0.0, 0.0)
        glEnd()

        glPopMatrix()
    
def renderSpawn (pos):
    fl = [-0.5,-0.5,0]
    fr = [0.5,-0.5,0]
    bl = [-0.5,0.5,0]
    br = [0.5,0.5,0]
    blu = [-0.5,0.5,0.3]
    bru = [0.5,0.5,0.3]
    back_bitl = [-0.4,0.4,0]
    back_bitr = [0.4,0.4,0]
    nor_left = [-1,0,0]
    nor_right = [1,0,0]
    nor_back = [0,1,0]
    nor_base = [0,0,1]
    nor_left_slope  = normalized_cross (subtract(back_bitl,fl),subtract(bl,back_bitl))
    nor_right_slope = normalized_cross (subtract(br,back_bitr),subtract(back_bitr,fr))
    nor_back_slope = normalized_cross (subtract(bru,back_bitl),subtract(blu,bru))
    
    glBegin(GL_TRIANGLES)
    glNormal3f(*nor_left)
    glVertex3f(*fl)
    glVertex3f(*blu)
    glVertex3f(*bl)
    glNormal3f(*nor_right)
    glVertex3f(*fr)
    glVertex3f(*br)
    glVertex3f(*bru)

    glNormal3f(*nor_left_slope)
    glVertex3f(*fl)
    glVertex3f(*back_bitl)
    glVertex3f(*blu)
    
    glNormal3f(*nor_right_slope)
    glVertex3f(*bru)
    glVertex3f(*back_bitr)
    glVertex3f(*fr)

    glNormal3f(*nor_back_slope)
    glVertex3f(*bru)
    glVertex3f(*blu)
    glVertex3f(*back_bitl)

    glVertex3f(*bru)
    glVertex3f(*back_bitl)
    glVertex3f(*back_bitr)

    glVertex3f(*bru)
    glVertex3f(*blu)
    glVertex3f(*back_bitr)
    
    glNormal3f(*nor_back)
    glVertex3f(*bl)
    glVertex3f(*blu)
    glVertex3f(*bru)
    
    glNormal3f(*nor_back)
    glVertex3f(*bl)
    glVertex3f(*bru)
    glVertex3f(*br)

    
    
    glEnd()
    
def generateDisplayList(renderingFunc):
    """
    Generates a display list.
    Rendering func can be a functor, this allows
    you to generate state before passing it in.
    """
    listID = glGenLists(1)
    glNewList(listID, GL_COMPILE)
    renderingFunc()
    glEndList()
    return listID

def callDisplayList(listID):
    glCallList(listID)

def draw_dot (pos):
    #print "drawing",pos
    glBegin (GL_POINTS)
    glVertex3f (*pos)
    glEnd ()
def draw_text (box ,str, font = GLUT_BITMAP_8_BY_13):
    """ Assumes the model view and projection view are set to map to pixels """
    glDisable (GL_TEXTURE_2D)
    glDisable(GL_TEXTURE_2D)
    chars = 0
    if len(box) == 4:
        capacity = (box[2] - box[0]) / 9
    cursor = box[0:2]
    glRasterPos2fv(cursor)
    for word in re.findall("\S+[\s]{0,1}", str):
        if len(box) == 4 and len(word) + chars > capacity:
            chars = 0
            cursor[1] -= 12
            glRasterPos2fv(cursor)
        for char in word:
            glutBitmapCharacter(font, ord(char))
            chars += 1
    glEnable (GL_TEXTURE_2D)
    