"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
import texture_manager
import animators

class MapElement(object):
    def __init__(self):
        # Tile placeholders 
        self.left = None
        self.right = None
        self.top = None
        self.bottom = None
        self.objects = []
        self.enemies = 0
        self.selected = False

    def execute (self):
        pass
    def complete (self):
        pass
    def executable (self):
        return False
    def stand_on_action (self):
        pass
    def is_deadly(self):
        return False
    def is_inside(self, pos):
        x,y,z = pos
        #print "wp:",pos
        selfx,selfy = self.pos
        size = self.size
        if x > selfx-(size/2) and x < selfx+(size/2) and y > selfy-(size/2) and y < selfy+(size/2):
            return True
        return False

    def is_goal(self):
        return False
    def do_setup(self, level):
        pass
    def habitable(self):
        return True
    def is_interesting (self):
        return False
    def push(self):
        pass
    def compile(self, level):
        pass
    def hack(self):
        pass
    def finalize(self):
        pass
class Tile(MapElement):
    def __init__(self,pos):
        MapElement.__init__(self)
        #self.anim = tile_animator(self,"floor1",pos)
        self.anim = animators.null_animator(self)
        self.pre_anim = animators.tile_animator(self,"floor1",pos)
        
        self.highlight = False
        self.command = None
        
    def habitable(self):
        return True
    def executable(self):
        return False
    def compile(self, level):
        pass

class TokenLocation(MapElement):
    def __init__(self,pos):
        MapElement.__init__(self)
        self.anim = animators.token_animator(self,"floor1","Forward2Token",pos)
        self.pre_anim = animators.null_animator(self)
    def habitable(self):
        return True
    def executable(self):
        return False
    def is_goal(self):
        return True
        
class Wall(MapElement):
    def __init__(self, pos):
        MapElement.__init__(self)
        #self.anim = animators.wall_animator(self,"mur040", pos) 
        self.anim = animators.null_animator(self)
        self.pre_anim = animators.wall_animator(self,"mur040", pos) 
    def compile(self, level):
        pass
    def habitable(self):
        return False
    def finalize(self):
        sides = [True,True,True,True]
        if type(self) == type(self.right):
            sides[0] = False
        if type(self) == type(self.top):
            sides[1] = False
        if type(self) == type(self.left):
            sides[2] = False
        if type(self) == type(self.bottom):
            sides[3] = False
        self.pre_anim.sides = sides
       
class Gate(MapElement):
    def __init__(self, pos,color):
        MapElement.__init__(self)
        self.anim = animators.gate_animator(self,"metal2", pos,color) 
        self.pre_anim = animators.null_animator(self)
        self.is_open = False
        self.new_state = False
        self.old_state = False
        self.full_time = 15
        self.time_left = 0
        self.color = color
    def compile(self, level):
        pass
    def habitable(self):
        return self.is_open
        
    def set_open (self, state): #should be called from within an execute of a robot
        self.time_left = self.full_time
        self.new_state = state
    def executable(self):
        return True
    def execute (self):
        if (self.time_left != -1 and self.is_open):
            self.time_left -= 1
            if (self.time_left == 0):
                self.new_state = False
        pass
    def complete (self):
        #we have to open it here because we don't whether execute will happen before after the execute of a robot that stands on it
        self.old_state = self.is_open
        self.is_open = self.new_state
        
    def is_open (self):
        return self.is_open

class Switch(MapElement):

    def __init__(self,pos,color):
        MapElement.__init__(self)
        #self.anim = tile_animator(self,"switch",pos)
        self.anim = animators.null_animator(self)
        self.pre_anim = animators.switch_animator(self,pos,color)
        self.gates = []
        self.hacked = False
    def habitable(self):
        return True
    def executable(self):
        return True
    def add_gate (self,g):
        self.gates.append (g)
    def open_gates (self):
        for g in self.gates:
            g.set_open(True ) 
    
    def push (self):
        self.open_gates ()
    def is_interesting (self):
        return True
    def execute (self):
        if (self.hacked):
            self.open_gates ()
    def hack (self):
        self.hacked = True
        self.open_gates ()
        return True

LEFT = 1
RIGHT = 2
TOP = 4
BOTTOM = 8

lava_map =None

def get_texture(input_id):
    global lava_map
    if not lava_map:
        # not initialised yet.
        lava_map = {}
        lava_map[0] = "single_lava"
        lava_map[LEFT] = "left_lava"
        lava_map[RIGHT] = "right_lava"
        lava_map[LEFT | RIGHT] = "left_right_lava"
        lava_map[TOP] = "top_lava"
        lava_map[TOP | BOTTOM] = "top_bottom_lava"
        lava_map[TOP | RIGHT] = "top_right_lava"
        lava_map[TOP | LEFT] = "top_left_lava"
        lava_map[TOP | LEFT | RIGHT] = "top_left_right_lava"
        lava_map[BOTTOM] = "bottom_lava"
        lava_map[BOTTOM | LEFT] = "bottom_left_lava"
        lava_map[BOTTOM | RIGHT] = "bottom_right_lava"
        lava_map[BOTTOM | LEFT | RIGHT] = "left_right_bottom_lava"
        lava_map[BOTTOM | LEFT | TOP] = "top_left_bottom_lava"
        lava_map[BOTTOM | RIGHT | TOP] = "top_right_bottom_lava"
        lava_map[BOTTOM | LEFT | RIGHT | TOP] = "surrounded_lava"
    return lava_map[input_id]

class Lava(MapElement):
    def __init__(self, pos):
        MapElement.__init__(self)
        self.anim = animators.null_animator(self)
        self.pre_anim = animators.tile_animator(self,"surrounded_lava",pos)
        self.pos = pos
        self.pre_anim = animators.null_animator(self)
    def compile(self, level):
        texture_select = 0
        if type(self) == type(self.left):
            texture_select |= LEFT
        if type(self) == type(self.right):
            texture_select |= RIGHT
        if type(self) == type(self.top):
            texture_select |= TOP
        if type(self) == type(self.bottom):
            texture_select |= BOTTOM
        texture_name = get_texture(texture_select)
        self.pre_anim = animators.tile_animator(self, texture_name, self.pos)
    def is_deadly(self):
        return True

class Spawn(MapElement):
    def __init__(self,pos, dir):
        MapElement.__init__(self)
        self.anim = animators.spawn_animator(self,"metal2","floor1", pos) 
        self.pre_anim = animators.null_animator(self)
        self.pos = pos[:-1]
        self.size = 1.0
        self.selected = False
        self.commands = None
        self.activated = False
        self.dir = dir
    
    def do_setup(self, level):
        level.renderableList.append(self)
        level.addSpawnPoint(self)
    def push (self):
        #print "pushed"
        self.activated = True
    def enable (self):
        self.activated = True
    def is_interesting(self):
        return True
